﻿CLootMaster	= LibStub("AceAddon-3.0"):NewAddon("CouncilLootMaster", "AceConsole-3.0", "AceComm-3.0", "AceEvent-3.0", "AceTimer-3.0", "AceHook-3.0")

local version 	    			= 'r36'
local iVersion	    			= 36
local lastCompatibleVersion 	= 33
local _G            			= _G

local debug         = false
local addon         = CLootMaster		-- Local instance of the addon

local L = LibStub("AceLocale-3.0"):GetLocale("CouncilLootMaster")

local usage = [=[

This mod provides a full loot council system based on some submodules from the EPGPLootMaster addon.
USAGE: Create a party/raid, setup a master looter. When master loot drops, just click the item and the UI will open.
Commandline Tools:
/cloot version: Show the versionchecker
/cloot config: Show the options window
/cloot reset: Resets window locations and scale
/cloot hide: Manually hide the Master Looter UI
/cloot show: Manually show the Master Looter UI
/cloot toggle: Manually toggle between showing and hiding the Master Looter UI
]=]

--[[
	Returns a table serialized as a string.
]]
function tprint( data, level )
	level = level or 0
	local ident=strrep('    ', level)
	if level>5 then return end

	if type(data)~='table' then print(tostring(data)) end

	for index,value in pairs(data) do repeat
		if type(value)~='table' then
			print( ident .. '['..index..'] = ' .. tostring(value) .. ' (' .. type(value) .. ')' )
			break
		end
		print( ident .. '['..index..'] = {')
		tprint(value, level+1)
		print( ident .. '}' )
	until true end
end

--[[
	Returns current version string
]]
function CLootMaster:GetVersionString()
	return self.version
end

--[[
	Checks whether a unit is in raid.
	Put in since patch 5.4.2 now puts people's server after their names even on own servers.
]]
function CLootMaster:UnitInRaid(name)
	local short_name = strsplit("-", name)
	return UnitInRaid(short_name) or UnitInRaid(name)
end

--[[
	Initialization function
]]
function CLootMaster:OnInitialize()

	self.version 	= version
	self.iVersion 	= iVersion

	self.db = LibStub("AceDB-3.0"):New("CouncilLootMaster")

	self.db:RegisterDefaults(
		{
			profile = {

				auto_announce_threshold = 4,
				AutoLootThreshold = 2,
				hideOnSelection = true,
				loot_timeout = 60,
				filterCouncilLootmasterMessages = true,

				votingWatchAllowed = false,
				votingDisallowSelf = false,
				votingSendGuildRank = true,
				votingSendGuildRankList = {true},
				votingSendAssistantOnly = false,

				monitor = true,
				monitorSend = true,
				monitorSendGuildRank = true,
				monitorSendGuildRankList = {true},
				monitorSendAssistantOnly = false,
				monitorThreshold = 2,
				ignoreResponseCorrections = false,
				hideMLOnCombat = false,
				hideSelectionOnCombat = false,
				allowCandidateNotes = true,
				monitorIncomingThreshold = 3,
				audioWarningOnSelection = true,
				use_councillootmaster = 'ask'
			}
		}
	)

	self.lootList   = {}
	self.lootMLCache = {}

	-- Client responses, DO NOT CHANGE ORDER!
	self.RESPONSE = {
		{ ["CODE"]      = "NOTANNOUNCED",   ["SORT"] =  100,  ["COLOR"] = {1,1,1},        ["TEXT"] = L['Not yet announced to candidate'] },
		{ ["CODE"]      = "INIT",           ["SORT"] =  200,  ["COLOR"] = {1,0,0},        ["TEXT"] = L['Offline or lootmaster not installed?'] },
		{ ["CODE"]      = "WAIT",           ["SORT"] =  300,  ["COLOR"] = {1,0.5,0},      ["TEXT"] = L['Making selection, please wait...'] },
		{ ["CODE"]      = "TIMEOUT",        ["SORT"] =  400,  ["COLOR"] = {1,0,1},        ["TEXT"] = L['Candidate did not respond on time.'] },
		{ ["CODE"]      = "MAIN",           ["SORT"] =  500,  ["COLOR"] = {0,1,0},    	  ["TEXT"] = 'Mainspec' },
		{ ["CODE"]      = "OFF",          	["SORT"] =  600,  ["COLOR"] = {1,0.65,0},    ["TEXT"] = 'Offspec' },
		{ ["CODE"]      = "PASS",           ["SORT"] = 	700,  ["COLOR"] = {0.6,0.6,0.6},  ["TEXT"] = L['Pass'] }
	}
	for i,d in ipairs(self.RESPONSE) do
		self.RESPONSE[d.CODE] = i
		self.RESPONSE[d.CODE .. "_TEXT"] = d.TEXT
	end
	CLootMaster.RESPONSE = self.RESPONSE

	-- Loot receive types.
	self.LOOTTYPE = {
		{ ["CODE"]      = "UNKNOWN",        ["TEXT"] = L['%s received %s for unknown reason%4$s.'] },
		{ ["CODE"]      = "VOTE",           ["TEXT"] = L['%s received %s.'] },
		{ ["CODE"]      = "DISENCHANT",     ["TEXT"] = L['%s received %s for disenchantment%4$s.'] }
	}
	for i,d in ipairs(self.LOOTTYPE) do
		self.LOOTTYPE[d.CODE] = i
		self.LOOTTYPE[d.CODE .. "_TEXT"] = d.TEXT
	end
	CLootMaster.LOOTTYPE = self.LOOTTYPE

	-- Register communications
	self:RegisterComm("CLootMasterC",	"CommandReceived")

	-- Register communications for version checking
	self:RegisterComm("CouncilLMVChk",			"CommVersionCheckRequest")
	self:RegisterComm("CouncilLMVRsp",			"CommVersionCheckResponse")

	--[===[@non-alpha@
	-- Check for updated versions in the guild
	if IsInGuild() then
		self:SendCommMessage("CouncilLMVChk",      lastCompatibleVersion .. "_" .. iVersion .. "_" .. version, "GUILD")
	end
	--@end-non-alpha@]===]

	self:RegisterEvent("PLAYER_REGEN_DISABLED", "EnterCombat")
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "LeaveCombat")

	self:RegisterEvent("PLAYER_ENTERING_WORLD", "EnterWorld")

	self:RegisterChatCommand("cloot",	        "SlashHandler")
end

--[[
	Register addon prefixes
]]
function CLootMaster:EnterWorld()
	RegisterAddonMessagePrefix("CLootMasterC")
	RegisterAddonMessagePrefix("CouncilLMVChk")
	RegisterAddonMessagePrefix("CouncilLMVRsp")
	RegisterAddonMessagePrefix("CLootMasterML")
end

--[[
	Delays localizing loot types.
]]
function CLootMaster:OnEnable()
	-- We don't need these right away, so localize them after a 10 second delay.
	self:ScheduleTimer("LocalizeLootTypes", 10)
end

--[[
	Slash-command handling function, takes the following arguments:

	- version/versioncheck/vc/v			version checking
	- debug								toggles debugging
	- verbose							toggles verbose debugging
	- reset								resets window scale and window locations
	- config/c/options/o				opens the options menu
	- close/hide						hides the council window
	- open/show							(re)opens the council window
	- toggle							toggles the council window
	- add/announce						announce a piece of loot
	TODO: finish this list and evaluate what's needed and what can be removed
]]
function CLootMaster:SlashHandler( input )
	local _,_,command, args = string.find( input, "^(%a-) (.*)$" )
	command = command or input

	if command=='version' or command=='versioncheck' or command=='vc' or command=='v' then

		self:ShowVersionCheckFrame()

	elseif command=='debug' then

		self.debug = not self.debug
		if self.debug then
			self:Print('Debugging enabled')
		else
			self:Print('Debugging disabled')
		end

	elseif command=='verbose' then

		self.verbose = not self.verbose
		if self.verbose then
			self.debug = true
			self:Print(L['Verbose debugging enabled'])
		else
			self.debug = false
			self:Print(L['Verbose debugging disabled'])
		end

	elseif command=='reset' then

		CLootMaster:SetUIScale(1.0)
		if CLootMasterML then
			CLootMasterML:SetUIScale(1.0)
		end

	elseif command=='config' or command=='c' or command=='options' or command=='o' then

		if CLootMasterML then
			InterfaceOptionsFrame_OpenToCategory('CouncilLootMaster')
			InterfaceOptionsFrame_OpenToCategory('CouncilLootMaster') -- silly fix to actually open to the category as well (courtesy of atlasloot)
		else
			self:Print('Council module not enabled, options unavailable')
		end

	elseif command=='close' or command=='hide' then

		if CLootMasterML then
			self:Print('Hiding council window, reopen with /cloot show')
			CLootMasterML.Hide(CLootMasterML)
		end

	elseif command=='open' or command=='show' then

		if CLootMasterML then
			CLootMasterML.Show(CLootMasterML)
		end

	elseif command=='toggle' then

		if CLootMasterML then
			if CLootMasterML.IsShown(CLootMasterML) then
				CLootMasterML.Hide(CLootMasterML)
			else
				CLootMasterML.Show(CLootMasterML)
			end
		end

	elseif command=='debuglog' or command=='log' or command=='lootlog' then

		self:OutputLog()

	elseif command=='logtest' then

		for i = 1, 20 do
			local item = GetInventoryItemLink("player",i)


			if item then
				local itemName = tostring(GetItemInfo(item))
				local itemID = tostring(CLootMasterML:GetItemIDFromLink(item))
				local entry = self:CreateLogEntry()
				entry.name=itemID..':'..itemName
				entry.slots={}
				for j = 1, 20 do
					local litem = GetInventoryItemLink("player",j)

					if litem then
						local litemName = tostring(GetItemInfo(litem))
						local litemID = tostring(CLootMasterML:GetItemIDFromLink(litem))
						entry.slots[j] = litemID..':'..litemName
					else
						entry.slots[j] = 'empty'
					end

				end
			end
		end

		self:OutputLog()
		self:Print('done')

	else

		self:Print( format('%s loaded.', version) .. usage )

	end
end

--[[
	Returns RGB values for the given hex color code
]]
function CLootMaster:ColorHexToRGB(color)
	color = tostring(color)
	local r,g,b = strmatch(color,'(%x%x)(%x%x)(%x%x)')
	r = tonumber(format('0x%s', r or 'ff'))/255
	g = tonumber(format('0x%s', g or 'ff'))/255
	b = tonumber(format('0x%s', b or 'ff'))/255
	return r,g,b
end

--[[
	Returns hex color code for the given RGB values
]]
function CLootMaster:ColorRGBToHex(r,g,b)
	r = tonumber(r) or 1
	g = tonumber(g) or 1
	b = tonumber(b) or 1
	return format('%02x%02x%02x',floor(r*255), floor(g*255), floor(b*255))
end


--Data for the GetGearByINVTYPE function
local INVTYPE_Slots = {
	INVTYPE_HEAD		    = "HeadSlot",
	INVTYPE_NECK		    = "NeckSlot",
	INVTYPE_SHOULDER	    = "ShoulderSlot",
	INVTYPE_CLOAK		    = "BackSlot",
	INVTYPE_CHEST		    = "ChestSlot",
	INVTYPE_WRIST		    = "WristSlot",
	INVTYPE_HAND		    = "HandsSlot",
	INVTYPE_WAIST		    = "WaistSlot",
	INVTYPE_LEGS		    = "LegsSlot",
	INVTYPE_FEET		    = "FeetSlot",
	INVTYPE_SHIELD		    = "SecondaryHandSlot",
	INVTYPE_ROBE		    = "ChestSlot",
	INVTYPE_2HWEAPON	    = {"MainHandSlot","SecondaryHandSlot"},
	INVTYPE_WEAPONMAINHAND	= "MainHandSlot",
	INVTYPE_WEAPONOFFHAND	= {"SecondaryHandSlot",["or"] = "MainHandSlot"},
	INVTYPE_WEAPON		    = {"MainHandSlot","SecondaryHandSlot"},
	INVTYPE_THROWN		    = {"MainHandSlot"},
	INVTYPE_RANGED		    = {"MainHandSlot"},
	INVTYPE_RANGEDRIGHT 	= {"MainHandSlot"},
	INVTYPE_FINGER		    = {"Finger0Slot","Finger1Slot"},
	INVTYPE_HOLDABLE	    = {"SecondaryHandSlot", ["or"] = "MainHandSlot"},
	INVTYPE_TRINKET		    = {"TRINKET0SLOT", "TRINKET1SLOT"}
}

--[[
	Extract the itemlinks, itemlevel and texture of the players current equipment for
	the given inventory slot.
]]
function CLootMaster:GetGearByINVTYPE( INVTYPE, unit )

	if not unit then unit="player" end

	if not INVTYPE_Slots[INVTYPE] then return '' end
	local ret = {}
	local slot = INVTYPE_Slots[INVTYPE]

	local item = GetInventoryItemLink(unit,GetInventorySlotInfo(slot[1] or slot))
	if not item and slot['or'] then
		item = GetInventoryItemLink(unit,GetInventorySlotInfo(slot['or']))
	end
	if item then tinsert(ret, item) end
	if slot[2] then
		item = GetInventoryItemLink(unit,GetInventorySlotInfo(slot[2]))
		if item then tinsert(ret, item) end
	end
	for i, item in ipairs(ret) do
		local _, _, _, ilevel, _, _, _, _, _, itemTexture = GetItemInfo(item)
		ret[i] = format('%s^%s^%s', item, ilevel or -1, itemTexture)
	end
	return strjoin('$', unpack(ret))
end

--[[
	Tries to extract BoP, BoE or BoU status for item
]]
function CLootMaster:GetItemBinding(item)
	if not item then return end
	if not self.bindingtooltip then
		self.bindingtooltip = CreateFrame("GameTooltip", "CLootMasterBindingTooltip", UIParent, "GameTooltipTemplate")
	end
	local tip = self.bindingtooltip
	tip:SetOwner(UIParent, "ANCHOR_NONE")
	tip:SetHyperlink(item)
	local numLines = CLootMasterBindingTooltip:NumLines()
	if numLines>6 then numLines=6 end
		for i=1, numLines do
			local line = _G['CLootMasterBindingTooltipTextLeft' .. i]
			if line and line.GetText then
				local t = line:GetText()
				if t == ITEM_BIND_ON_PICKUP then
				tip:Hide()
				return "pickup"
			elseif t == ITEM_BIND_ON_EQUIP then
				tip:Hide()
				return "equip"
			elseif t == ITEM_BIND_ON_USE then
				tip:Hide()
				return "use"
			end
		end
	end
	tip:Hide()
	return nil
end

-- Default english locale, this will automatically get updated by the
-- UpdateClassTranslator function, called from other places in the CouncilLM package.
local classLocalizeTable = {
	['MAGE']            = 'Mage',
	['WARRIOR']         = 'Warrior',
	['DEATHKNIGHT']     = 'Death Knight',
	['WARLOCK']         = 'Warlock',
	['DRUID']           = 'Druid',
	['SHAMAN']          = 'Shaman',
	['ROGUE']           = 'Rogue',
	['PRIEST']          = 'Priest',
	['PALADIN']         = 'Paladin',
	['HUNTER']          = 'Hunter',
	['MONK']       		= 'Monk'
}

-- Build the reverse lookup tables and the bit encoding table.
local classUnlocalizeTable = {}
local classBitTable = {}
local classCount = 0
local bit_bor = bit.bor
local bit_band = bit.band
for u, l in pairs(classLocalizeTable) do
	classUnlocalizeTable[l] = u
	classBitTable[u] = 2^classCount
	classCount = classCount + 1
end

-- Table of eligible classes for each loot type (bar offhands)
local eligibleTable = {
	['One-Handed Axes']     = {'DEATHKNIGHT','PALADIN','SHAMAN','WARRIOR','ROGUE','MONK'},
	['One-Handed Maces']    = {'ROGUE','PRIEST','SHAMAN','DRUID','DEATHKNIGHT','PALADIN','WARRIOR'},
	['One-Handed Swords']   = {'DEATHKNIGHT','PALADIN','WARLOCK','MAGE','MONK','ROGUE','WARRIOR'},
	['Daggers']             = {'DRUID','MAGE','PRIEST','ROGUE','SHAMAN','WARLOCK'},
	['Fist Weapons']        = {'DRUID','MONK','ROGUE','SHAMAN','WARRIOR'},
	['Wands']               = {'MAGE','PRIEST','WARLOCK'},
	['Two-Handed Axes']     = {'DEATHKNIGHT','PALADIN','SHAMAN','WARRIOR'},
	['Two-Handed Maces']    = {'DEATHKNIGHT','DRUID','PALADIN','SHAMAN','WARRIOR'},
	['Two-Handed Swords']   = {'DEATHKNIGHT','PALADIN','WARRIOR'},
	['Polearms']            = {'DEATHKNIGHT','MONK','PALADIN','WARRIOR','DRUID'},
	['Staves']              = {'DRUID','MAGE','MONK','PRIEST','SHAMAN','WARLOCK','WARRIOR'},
	['Bows']                = {'HUNTER'},
	['Crossbows']           = {'HUNTER'},
	['Guns']                = {'HUNTER'},
	['Shields']             = {'WARRIOR','PALADIN','SHAMAN'},
	['Cloth']				= {'MAGE','WARLOCK','PRIEST'},
	['Leather']             = {'ROGUE','DRUID','MONK'},
	['Mail']                = {'HUNTER','SHAMAN'},
	['Plate']               = {'DEATHKNIGHT','PALADIN','WARRIOR'}
}

-- Separate list of offhands since they dont have their own itemSubType
local eligibleForOffhands = {'DRUID','MAGE','MONK','PALADIN','PRIEST','SHAMAN','WARLOCK'}

-- Table of eligible classes per stat
local eligibleStatTable = {
	['ITEM_MOD_SPIRIT_SHORT'] 		= {'DRUID','MONK','PALADIN','PRIEST','SHAMAN'},
	['ITEM_MOD_AGILITY_SHORT'] 		= {'DRUID','HUNTER','MONK','ROGUE','SHAMAN'},
	['ITEM_MOD_STRENGTH_SHORT'] 	= {'DEATHKNIGHT','PALADIN','WARRIOR'},
	['ITEM_MOD_INTELLECT_SHORT'] 	= {'DRUID','MAGE','MONK','PALADIN','PRIEST','SHAMAN','WARLOCK'}
}

-- Separate table for trinkets (mostly for tanking stat reasons)
local eligibleExtraStatTable = {
	['ITEM_MOD_PARRY_RATING_SHORT'] = {'DEATHKNIGHT','PALADIN','WARRIOR'},
	['ITEM_MOD_DODGE_RATING_SHORT'] = {'DEATHKNIGHT','DRUID','MONK','PALADIN','WARRIOR'},
	['ITEM_MOD_STAMINA_SHORT'] 		= {'DEATHKNIGHT','DRUID','MONK','PALADIN','WARRIOR'},
	['ITEM_MOD_SPIRIT_SHORT'] 		= {'DRUID','MONK','PALADIN','PRIEST','SHAMAN'},
	['ITEM_MOD_AGILITY_SHORT'] 		= {'DRUID','HUNTER','MONK','ROGUE','SHAMAN'},
	['ITEM_MOD_STRENGTH_SHORT'] 	= {'DEATHKNIGHT','PALADIN','WARRIOR'},
	['ITEM_MOD_INTELLECT_SHORT'] 	= {'DRUID','MAGE','MONK','PALADIN','PRIEST','SHAMAN','WARLOCK'}
}

-- Table order, for the normal table start iterating at 4
local eligibleTableOrder = {'ITEM_MOD_PARRY_RATING_SHORT','ITEM_MOD_DODGE_RATING_SHORT','ITEM_MOD_STAMINA_SHORT','ITEM_MOD_SPIRIT_SHORT','ITEM_MOD_AGILITY_SHORT','ITEM_MOD_STRENGTH_SHORT','ITEM_MOD_INTELLECT_SHORT'}

-- Table to link localized stat name to the global string for that stat
local extraStatTranslator = {
	[ITEM_MOD_AGILITY_SHORT] = 'ITEM_MOD_AGILITY_SHORT',
	[ITEM_MOD_INTELLECT_SHORT] = 'ITEM_MOD_INTELLECT_SHORT',
	[ITEM_MOD_SPIRIT_SHORT] = 'ITEM_MOD_SPIRIT_SHORT',
	[ITEM_MOD_STRENGTH_SHORT] = 'ITEM_MOD_STRENGTH_SHORT',
	[ITEM_MOD_DODGE_RATING_SHORT] = 'ITEM_MOD_DODGE_RATING_SHORT',
	[ITEM_MOD_PARRY_RATING_SHORT] = 'ITEM_MOD_PARRY_RATING_SHORT',
	[ITEM_MOD_STAMINA_SHORT] = 'ITEM_MOD_STAMINA_SHORT'
}

-- Make the lookup table for localized subTypes.
local subTypeLocalized = {}
for l, _ in pairs(eligibleTable) do subTypeLocalized[l]=l end

--[[
	Check whether a given item is eligible for a given player class
]]
function CLootMaster:EligibleItemCheck( itemLink, playerLocalClass, playerClass )

	if not itemLink then return end

	if not self.bindingtooltip then
		self.bindingtooltip = CreateFrame("GameTooltip", "CLootMasterBindingTooltip", UIParent, "GameTooltipTemplate")
	end
	local tip = self.bindingtooltip
	tip:SetOwner(UIParent, "ANCHOR_NONE")
	tip:SetHyperlink(itemLink)

	local tooltipTextCache = {}
	-- lets see if we can find a 'Classes: Mage, Druid' string on the itemtooltip
	-- just scan all the lines.
	for i = 1, CLootMasterBindingTooltip:NumLines() do
		local linetext = _G["CLootMasterBindingTooltipTextLeft" .. i]
		local text = linetext:GetText()

		-- save tooltip text for possible later use with trinkets
		tinsert(tooltipTextCache, text)

		local localizedClasses = gsub( text or '', ', ', ',' )
		localizedClasses = localizedClasses:match( gsub(ITEM_CLASSES_ALLOWED,"%%s","(.*)") )

		if localizedClasses then
			if string.find(localizedClasses,playerLocalClass) then
				-- Yep, this item is available for this class
				tip:Hide()
				return true
			else
				-- some classes were found but it wasn't the one we're checking now!
				tip:Hide()
				return false
			end
		end
	end

	tip:Hide()

	-- item type check
	local _, _, _, _, _, _, itemSubType, _, itemEquipLoc = GetItemInfo(itemLink)

	local typeCheckPassed = false

	local itemRecognized = false

	if itemEquipLoc and itemEquipLoc=='INVTYPE_HOLDABLE' then
		itemRecognized = true
		for _,offhClass in ipairs(eligibleForOffhands) do
			if offhClass==playerClass then
				typeCheckPassed = true
			end
		end
	elseif itemEquipLoc and (itemEquipLoc=='INVTYPE_NECK' or itemEquipLoc=='INVTYPE_BODY' or itemEquipLoc=='INVTYPE_FINGER' or itemEquipLoc=='INVTYPE_TRINKET' or itemEquipLoc=='INVTYPE_CLOAK' or itemEquipLoc=='INVTYPE_TABARD') then
		itemRecognized = true
		typeCheckPassed = true
	elseif itemSubType then
		local eligibleClassArray = eligibleTable[ subTypeLocalized[itemSubType] ]
		if eligibleClassArray then
			itemRecognized = true
			-- There is a list of classes for this item type
			for _, class in ipairs(eligibleClassArray) do
				if class==playerClass then
					typeCheckPassed = true
				end
			end
		end
	end

	-- item stat check
	if typeCheckPassed then
		if itemEquipLoc and itemEquipLoc=='INVTYPE_TRINKET' then
			local noStatsFound = false
			-- for trinkets first request stats, then scan the tooltip if nothing is found
			local statList = GetItemStats(itemLink)
			if statList then
				for i=1,#eligibleTableOrder do
					local currentStat = eligibleTableOrder[i]
					if statList[currentStat] then
						for _,class in ipairs(eligibleExtraStatTable[currentStat]) do
							if class==playerClass then
								return true
							end

						end
					end
				end
			else
				noStatsFound = true
			end
			local textMatch = false
			for _,text in ipairs(tooltipTextCache) do
				for k,v in pairs(extraStatTranslator) do
					if text:match(k) then
						textMatch = true
						for _,class in ipairs(eligibleExtraStatTable[v]) do
							if class==playerClass then
								return true
							end
						end
					end
				end
			end

			if noStatsFound and not textMatch then
				-- no match found through stats or tooltip for any class, assuming available for everyone
				return true
			end
		else
			-- for non-trinkets just request the stats
			local statList = GetItemStats(itemLink)
			if statList then
				for i=4,#eligibleTableOrder do
					local currentStat = eligibleTableOrder[i]
					if statList[currentStat] then
						for _,class in ipairs(eligibleStatTable[currentStat]) do
							if class==playerClass then
								return true
							end
						end
						return false -- there is a class-defining stat on this item but this class isn't eligible
					end
				end
				return true --item has stats but no relevant ones to any class? old item without primary stats?
			else
				return true --item has no stats, special item?
			end
		end
	elseif not itemRecognized then
		return true --item didn't fit any item type, assume everyone can use it
	end
	return false
end

--[[
	Returns the localized name for a given unlocalized class
]]
function CLootMaster:LocalizeClass( classFilename )
	return classLocalizeTable[classFilename]
end

--[[
	Returns the unlocalized name for a given localized class
]]
function CLootMaster:UnlocalizeClass( localizedClass )
	return classUnlocalizeTable[localizedClass]
end

--[[
	A function that updates the class localizers
]]
function CLootMaster:UpdateClassLocalizer( localizedClass, classFilename )
	classLocalizeTable[classFilename] = localizedClass
	classUnlocalizeTable[localizedClass] = classFilename
end

--[[
	Returns the encoded equilvalent of all classes being eligible
]]
function CLootMaster:FullClassesEncoded()
	local fullClassList = {
		['MAGE']            = true,
		['WARRIOR']         = true,
		['DEATHKNIGHT']     = true,
		['WARLOCK']         = true,
		['DRUID']           = true,
		['SHAMAN']          = true,
		['ROGUE']           = true,
		['PRIEST']          = true,
		['PALADIN']         = true,
		['HUNTER']          = true,
		['MONK']            = true,
	}
	
	return self:EncodeUnlocalizedClasses(fullClassList)
end

--[[
	Encode an array of unlocalized classes, used for communication purposes
]]
function CLootMaster:EncodeUnlocalizedClasses( classFilenameArray )
	if not classFilenameArray then return 0 end
	local bits = 0
	for class, bool in pairs(classFilenameArray) do
		if not classBitTable[class] then
			self:Print(format(L['Serious error in class bitencoder, class %s not found. Please make sure you have the latest version installed and report if problem persists.'], class or 'nil'))
			return 0
		end
		if bool then
			bits = bit_bor(bits, classBitTable[class])
		end
	end
	return bits
end

--[[
	Decode an array of unlocalized classes, used for communication purposes
]]
function CLootMaster:DecodeUnlocalizedClasses( encodedClassArray )
	encodedClassArray = tonumber(encodedClassArray) or 0
	if encodedClassArray==0 then return nil end
	local classes = {}
	for class, bits in pairs(classBitTable) do
		if bit_band(encodedClassArray, bits) == bits then
			classes[class] = true
			encodedClassArray = encodedClassArray - bits
		end
	end

	if encodedClassArray~=0 then
		self:Print(format(L['Serious error in class bitdecoder, bits %s not found. Please make sure you have the latest version installed and report if problem persists.'], tostring(encodedClassArray)))
		return nil
	end

	return classes
end

-- Table with items to localize itemsubtypes
local subTypeLocalizedLookup = {
	['One-Handed Axes']     = 'Hitem:31071',    -- Grom'tor's Charge
	['One-Handed Maces']    = 'Hitem:27901',    -- Blackout Truncheon
	['One-Handed Swords']   = 'Hitem:28267',    -- Edge of the Cosmos
	['Daggers']             = 'Hitem:30999',    -- Ashtongue Blade
	['Fist Weapons']        = 'Hitem:27747',    -- Boggspine Knuckles
	['Wands']               = 'Hitem:25640',    -- Nesingwary Safari Stick
	['Two-Handed Axes']     = 'Hitem:32663',    -- Apexis Cleaver
	['Two-Handed Maces']    = 'Hitem:30093',    -- Great Earthforged Hammer
	['Two-Handed Swords']   = 'Hitem:27769',    -- Endbringer
	['Polearms']            = 'Hitem:24044',    -- Hellreaver
	['Staves']              = 'Hitem:25760',    -- Battle Mage's Baton
	['Bows']                = 'Hitem:31072',    -- Lohn'goron, Bow of the Torn-heart
	['Crossbows']           = 'Hitem:28397',    -- Emberhawk Crossbow
	['Guns']                = 'Hitem:31000',    -- Bloodwarder's Rifle
	['Shields']             = 'Hitem:31491',    -- Netherwing Defender's Shield
	['Cloth']				= 'Hitem:31216',	-- Abyssal Cloth Footwraps
	['Leather']             = 'Hitem:31215',    -- Abyssal Leather Treads
	['Mail']                = 'Hitem:31214',    -- Abyssal Mail Greaves
	['Plate']               = 'Hitem:31213'     -- Abyssal Plate Sabatons
}
local localizeLootTypesCount = 0
local hasEnglishLocale = (GetLocale() == 'enUS')

--[[
	This function tries to localize the itemSubTypes used in GetItemAutoPassClasses()
	It tries to do so by looking up a few known items.
]]
function CLootMaster:LocalizeLootTypes()

	if not self.bindingtooltip then
		self.bindingtooltip = CreateFrame("GameTooltip", "CLootMasterBindingTooltip", UIParent, "GameTooltipTemplate")
	end
	local tip = self.bindingtooltip
	tip:SetOwner(UIParent, "ANCHOR_NONE")

	localizeLootTypesCount = localizeLootTypesCount + 1
	local failed = false
	for sType, item in pairs(subTypeLocalizedLookup) do
		local itemName, _, _, _, _, _, itemSubType, _, _, _ = GetItemInfo(item)
		if itemName and itemSubType then
			-- Sanitycheck when we have an enUS client. Check our output against the strings we already know.
			if hasEnglishLocale and itemSubType~=sType then
				self:Print(format('Failed looking up localized strings for itemSubTypes. Expected %s for %s, got %s. Please report!', tostring(sType), tostring(item), tostring(itemSubType)))
			else
				subTypeLocalized[itemSubType] = sType
			end
			localizeLootTypesCount = 0
			subTypeLocalizedLookup[sType] = nil
		else
			tip:SetHyperlink(item)
			failed = true
			-- stop the loop and try again after 2 secs
			break
		end
	end
	if failed then
		if localizeLootTypesCount<5 then
			self:ScheduleTimer("LocalizeLootTypes", 2)
		else
			self:Print(format('Failed looking up localized loottypes, bailing out permanently!'))
		end
	end
end

--[[
	Function that gets called when asked for our version
]]
function CLootMaster:CommVersionCheckRequest(prefix, message, distribution, sender)
	local _,_,lastCompVersion,senderVersionInt, senderVersionString = string.find(message, "^(%d+)_(%d+)_(.*)$")
	lastCompVersion = tonumber(lastCompVersion) or 0
	senderVersionInt = tonumber(senderVersionInt) or 0

	if (debug) then
		self:Print( string.format("VReq from %s, has version %s", sender, senderVersionString) )
	end
	if (senderVersionInt<iVersion) then
		-- Senders version has been outdated
		self:SendCommMessage("CouncilLMVRsp", lastCompatibleVersion .. "_" .. iVersion .. "_" .. version, "WHISPER", sender)
	end
	if (senderVersionInt>iVersion and not debug) then
		-- Our version is outdated
		if (lastCompVersion>iVersion) then
			self:ShowUrgentUpdateFrame( sender, iVersion, senderVersionString )
		else
			self:ShowUpdateFrame( sender, iVersion, senderVersionString )
		end
	end
end

--[[
	Function that captures version responses
]]
function CLootMaster:CommVersionCheckResponse(prefix, message, distribution, sender)
	local _,_,lastCompVersion,senderVersionInt, senderVersionString = string.find(message, "^(%d+)_(%d+)_(.*)$")
	lastCompVersion = tonumber(lastCompVersion) or 0
	senderVersionInt = tonumber(senderVersionInt) or 0

	if (debug) then
		self:Print( string.format("VResp from %s, has version %s", sender, senderVersionString) )
	end
	if senderVersionInt~=0 and self.versioncheckframe and self.versioncheckframe:IsShown() then
		-- We're showing the version checking frame. lets update it
		local memberID = self.versioncheckframe.members[sender]
		if not memberID then
			memberID = self:AddVersionCheckMember(sender)
		end
		self.versioncheckframe.rows[memberID]["cols"][2].value=senderVersionString
		self.versioncheckframe.rows[memberID]["cols"][3].value=senderVersionInt
		if self.versioncheckframe.rows[memberID]["start"] then
			self.versioncheckframe.rows[memberID]["cols"][4].value=GetTime() - self.versioncheckframe.rows[memberID]["start"]
		end
		self.versioncheckframe.sstScroll:SetData( self.versioncheckframe.rows )
		self.versioncheckframe.sstScroll:SortData()
		self.versioncheckframe.sstScroll:DoFilter()
	end
	if (senderVersionInt>iVersion and not debug) and senderVersionString:sub(1,1)=='v' then
		-- 	Our version is outdated
		if (lastCompVersion>iVersion) then
			self:ShowUrgentUpdateFrame( sender, iVersion, senderVersionString )
		else
			self:ShowUpdateFrame( sender, iVersion, senderVersionString )
		end
	end
end


--[DEBUG LOGGING STUFF]--
local debuglog = {}
function CLootMaster:LogSize()
	return #debuglog
end
function CLootMaster:RecurseLogOutput(entry)
	local output = {}
	local temp, itype
	for key, data in pairs(entry) do
		itype = type(data)
		if itype == 'table' then
			temp = self:RecurseLogOutput(data)
			itype = 't'
		elseif itype == 'number' then
			temp = data
			itype = 'i'
		else
			temp = format('"%s"',tostring(data))
		end
		tinsert(output, format('"%s":%s', key, temp))
	end
	return '{' .. strjoin(',', unpack(output)) .. '}'
end
function CLootMaster:OutputLog()
	local output = self:RecurseLogOutput(debuglog)

	StaticPopupDialogs["COUNCIL_LOGOUTPUT_POPUP"] = {
		text = 'CouncilLootmaster: please copy/paste the text below and put it up on the guild forum if you want, along with an explanation of the situation.',
		button1 = nil,
		button2 = 'OK',
		timeout = 0,
		whileDead = 1,
		--exclusive = 0,
		--showAlert = 0,
		hideOnEscape = 1,
		hasEditBox = 1,
		maxLetters = 0,
		OnShow = function(self)
			self.editBox:SetText(output)
			self.editBox:SetFocus()
			self.editBox:HighlightText()
		end,
		OnHide = function(self)
			if ( ChatFrameEditBox:IsShown() ) then
				ChatFrameEditBox:SetFocus()
			end
			self.editBox:SetText("")
		end,
		EditBoxOnEscapePressed = function(self)
			self:GetParent():Hide()
			ClearCursor()
		end
	}
	StaticPopup_Show("COUNCIL_LOGOUTPUT_POPUP")
end
function CLootMaster:CreateLogEntry()
	local entry = {
		["ts"]   = GetTime()
	}
	tinsert(debuglog, entry)
	while #debuglog>50 do
		tremove(debuglog, 1)
	end
	return entry
end
